#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>
#include <thread.h>
#include <cursor.h>
#include <mouse.h>
#include <keyboard.h>
#include <frame.h>
#include <plumb.h>
#include <html.h>
#include <url.h>
#include "dat.h"
#include "fns.h"

AUrl *
urlalloc(Runestr *src, Runestr *post, int m)
{
	AUrl *u;

	u = emalloc(sizeof(AUrl));
	copyrunestr(&u->src, src);
	if(m==HPost)
		copyrunestr(&u->post, post);
	u->method = m;
	incref(u);
	return u;
}

void
urlfree(AUrl *u)
{
	if(u==nil)
		return;

	if(decref(u)==0){
		closerunestr(&u->src);
		closerunestr(&u->act);
		closerunestr(&u->post);
		closerunestr(&u->ctype);
		free(u);
	}
}

AUrl *
urldup(AUrl *a)
{
	AUrl *b;

	b = emalloc(sizeof(AUrl));
	b->method = a->method;
	copyrunestr(&b->src, &a->src);
	copyrunestr(&b->act, &a->act);
	copyrunestr(&b->post, &a->post);
	copyrunestr(&b->ctype, &a->ctype);
	return b;
}

static 
Runestr
getattr(int conn, char *s)
{
	char buf[BUFSIZE];
	int fd, n;

	snprint(buf, sizeof(buf), "%s/%d/%s", webmountpt, conn, s);
	fd = open(buf, OREAD);
	if(fd < 0)
		error("can't open attr file");

	n = read(fd, buf, sizeof(buf)-1);
	if(n < 0)
		error("can't read");

	close(fd);
	buf[n] = '\0';
	return (Runestr){runesmprint("%s", buf), n};
}

int
urlopen(AUrl *u)
{
	char buf[BUFSIZE];
	int cfd, fd, conn, n;

	snprint(buf, sizeof(buf), "%s/clone", webmountpt);
	cfd = open(buf, ORDWR);
	if(cfd < 0)
		error("can't open clone file");

	n = read(cfd, buf, sizeof(buf)-1);
	if(n <= 0)
		error("reading clone");

	buf[n] = '\0';
	conn = atoi(buf);

	snprint(buf, sizeof(buf), "url %S", u->src.r);
	if(write(cfd, buf, strlen(buf)) < 0){
		fprint(2, "write: %s: %r\n", buf);
    Err:
		close(cfd);
		return -1;
	}
	if(u->method==HPost && u->post.r != nil){
		snprint(buf, sizeof(buf), "%s/%d/postbody", webmountpt, conn);
		fd = open(buf, OWRITE);
		if(fd < 0){
			fprint(2, "urlopen: bad query: %s: %r\n", buf);
			goto Err;
		}
		snprint(buf, sizeof(buf), "%S", u->post.r);
		if(write(fd, buf, strlen(buf)) < 0)
			fprint(2, "urlopen: bad query: %s: %r\n", buf);

		close(fd);
	}
	snprint(buf, sizeof(buf), "%s/%d/body", webmountpt, conn);
	fd = open(buf, OREAD);
	if(fd < 0){
		fprint(2, "open: %S: %r\n", u->src.r);
		goto Err;
	}
	u->ctype = getattr(conn, "contenttype");
	u->act = getattr(conn, "parsed/url");
	if(u->act.nr == 0)
		copyrunestr(&u->act, &u->src);

	close(cfd);
	return fd;
}

/* this is a HACK */
Rune *
urlcombine(Rune *b, Rune *u)
{
	Rune *p, *q, *sep, *s;
	int restore;
	Url *url, *burl;
	char *bc, *uc;
	Rune *r;

	if(u == nil)
		error("urlcombine: u == nil");

	if(validurl(u))
		return erunestrdup(u);

	if(b==nil || !validurl(b))
		error("urlcombine: b==nil || !validurl(b)");

	bc = smprint("%S",b);
	uc = smprint("%S",u);
	burl = parseurl(bc,nil);
	url = parseurl(uc, burl);
	free(bc);
	free(uc);
	r = runesmprint("%s", url->url);
	free(url);
	free(burl);
	return r;
	/*
	if(runestrncmp(u, L"//", 2) == 0){
		q =  runestrchr(b, L':');
		return runesmprint("%.*S:%S", (int)(q-b), b, u);
	}
	p = runestrstr(b, L"://");
	sep = L"";
	if(*u ==L'/' || *u==L'#' || *u==L'?')
		q = runestrchr(p+3, L'/');
	else{
		sep = L"/";
		restore = 0;
		s = runestrchr(p+3, L'?');
		if(s != nil){
			*s = '\0';
			restore = 1;
		}
		q = runestrrchr(p+3, L'/');
		if(restore)
			*s = L'?';
	}
	if(q == nil)
		return runesmprint("%S%S%S", b, sep, u);
	else
		return runesmprint("%.*S%S%S", (int)(q-b), b, sep, u);
	*/
}
